using System.Net.Http.Headers;
using System.Net.Http.Json;
using System.Security.Cryptography;
using System.Text;
using FitConnect.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;


public abstract class FunctionalBaseClass {
    protected readonly ILogger? logger;
    private RSA _rsa;
    public FitConnectEndpoints Endpoints { get; }

    protected FunctionalBaseClass(ILogger? logger, FitConnectEndpoints? endpoints) {
        Endpoints = endpoints ??
                    FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development);

        this.logger = logger;
    }


    /// <summary>
    /// Requesting an OAuth token from the FitConnect API.
    /// 
    /// You can get the Client ID and Client Secret from the FitConnect Self Service portal
    /// under https://portal.auth-testing.fit-connect.fitko.dev
    /// </summary>
    /// <param name="clientId">Your client Id</param>
    /// <param name="clientSecret">Your client Secret</param>
    /// <param name="scope">Scope if needed</param>
    /// <returns></returns>
    public async Task<OAuthAccessToken?> GetTokenAsync(string clientId, string clientSecret,
        string? scope = null) {
        var client = new HttpClient();
        client.DefaultRequestHeaders.Accept.Add(
            MediaTypeWithQualityHeaderValue.Parse("application/json"));

        var requestContent = new Dictionary<string, string> {
            { "grant_type", "client_credentials" },
            { "client_id", clientId },
            { "client_secret", clientSecret }
        };

        if (scope != null)
            requestContent["scope"] = scope;

        var content = new FormUrlEncodedContent(requestContent);

        var request = new HttpRequestMessage(HttpMethod.Post, Endpoints.TokenUrl) {
            Content = content,
            Method = HttpMethod.Post
        };

        var response = await client.SendAsync(request);

        return await response.Content.ReadFromJsonAsync<OAuthAccessToken>();
    }

    public Task<SecurityEventToken> GetSetDataAsync() {
        throw new NotImplementedException();
    }

    public byte[] EncryptDataAsync(byte[] data,
        byte[]? publicKey,
        string? password,
        int numberOfIterations) {
        _rsa = RSA.Create(2048);
        // _rsa.ImportRSAPublicKey(publicKey, out var read);

        logger?.LogInformation(
            "Encrypting data with public key: {}",
            Convert.ToBase64String(_rsa.ExportRSAPublicKey()));

        // var keyParams = new PbeParameters(
        //     PbeEncryptionAlgorithm.Aes256Cbc, HashAlgorithmName.SHA256, numberOfIterations);
        //
        // var privateKey =
        //     rsa.ExportEncryptedPkcs8PrivateKey(Encoding.UTF8.GetBytes(password), keyParams);
        //
        // logger?.LogInformation(
        //     "Private key: {}", privateKey);

        return _rsa.Encrypt(data, RSAEncryptionPadding.OaepSHA256);
    }

    public byte[] DecryptDataAsync(byte[] data, byte[] privateKey) {
        return _rsa.Decrypt(data, RSAEncryptionPadding.OaepSHA256);
    }


    private async Task<T?> RestCall<T>(Uri uri, HttpMethod method, string body) {
        var client = new HttpClient();
        client.DefaultRequestHeaders.Add("Accept", "application/json");
        client.DefaultRequestHeaders.Add("Content-Type", "application/json");

        var request = new HttpRequestMessage();
        request.Method = method;
        request.Content = new StringContent(body, Encoding.UTF8, "application/json");
        request.RequestUri = uri;

        var response = await client.SendAsync(request);

        if (response.IsSuccessStatusCode)
            return (await response.Content.ReadFromJsonAsync<T>());

        throw new HttpRequestException("Error calling FitConnect API");
    }
}
